#!/usr/bin/php -q
<?php
/* XMPP Roku SoundBridge Control Daemon
 * Version 1.0.0
 * Copyright 2006, Steve Blinch
 * http://code.blitzaffe.com
 * ============================================================================
 *
 * DETAILS
 *
 * Allows you to control and display messages on your Roku Soundbridge using
 * your favorite XMPP instant messaging client (such as Psi, Google Talk, etc.)
 *
 * This script currently allows you to do the following by sending commands
 * via XMPP:
 *	- display instant messages on your Soundbridge's display
 *  - receive realtime song information in your contact list (the status message
 *    for the daemon's XMPP account is automatically updated when the song
 *    changes)
 *	- perform basic control operations (stop playing, start playing, next
 *    track, previous track etc.) 
 *	- retrieve status information from the Roku SoundBridge
 *
 * Note that this script is is a daemon and is intended to be executed from
 * the console using te PHP CLI binary.  It should NOT be executed via a
 * webserver!
 *
 *
 * REQUIREMENTS
 *
 * This project requires three libraries which are also available at
 * http://code.blitzaffe.com:
 *
 * - PHP Jabber Class (used to communiate with your XMPP server)
 * - PHP Roku SoundBridge Interface Library (used to communicate with the
 *   Roku SoundBridge)
 * - HTTPRetriever (used by the SoundBridge interface library)
 * 
 *
 * USAGE
 *
 * To get this script up and running:
 *
 * 1. Create a new account on your XMPP server for your Roku SoundBridge to use.
 * 2. Edit config.php and enter your settings.  This should be more-or-less
 *    self-explanatory.
 * 3. Run: /path/to/php/php -q xmpprokud.php
 *
 * Now, simply fire up your usual instant messaging client and send a message
 * containing the word "help" to the account you created above.  If everything
 * is setup correctly, you should receive a reply listing all of the commands
 * you can send to the daemon.
 *
 *
 * LICENSE
 *
 * This script is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This script is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *	
 * You should have received a copy of the GNU General Public License along
 * with this script; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
require_once(dirname(__FILE__).'/config.php');
require_once('jabber/class_Jabber.php');
require_once('class_RokuSoundBridge.php');

define("RUN_TIME",-1);	// run indefinitely
define("CBK_FREQ",1);	// fire a callback event every second

// This class handles all events fired by the Jabber client class
class XMPPEventHandler {
	
	function XMPPEventHandler(&$xmpp) {
		$this->xmpp = &$xmpp;
		$this->soundbridge = &new RokuSoundBridge();
		
		echo "Connecting to server...\n";
		$this->beats = 0;
		$this->old_song_title = '';
	}
	
	// called when a connection to the Jabber server is established
	function handleConnected() {
		echo "Connected\n";
		
		// uncomment this as a quick way to register a new account on-the-fly
		// $this->xmpp->register(XMPP_USERNAME,XMPP_PASSWORD);
		
		// now that we're connected, tell the Jabber class to login
		$this->xmpp->login(XMPP_USERNAME,XMPP_PASSWORD,'SoundBridge');
	}
	
	// called after a login to indicate the the login was successful
	function handleAuthenticated() {
		echo "Authenticated\n";
		
		// browser for transport gateways and retrieve the roster; neither are
		// really necessary for this implementation, but what the hell :)
		$this->xmpp->browse();
		$this->xmpp->get_roster();
		
		// set our presence to the currently playing song
		echo "Setting initial presence...\n";
		$this->check_current_song();
		
		echo "Online and accepting commands via XMPP\n";
	}
	
	// called when a subscription request is received
	function handleSubscribe($packet) {

		$presence = &$packet["presence"]["@"];
		$from = $presence["from"];

		echo "Subscription request from $from\n";

		// accept the subscription automatically; this way users can subscribe to see our current
		// song information in our status message
		$this->xmpp->subscription_request_accept($from);
	}
	
	// called after a login to indicate that the login was NOT successful
	function handleAuthFailure($code,$error) {
		echo "Authentication failure: $error ($code)\n";
		
		// set terminated to TRUE in the Jabber class to tell it to exit
		$this->xmpp->terminated = true;
	}
	
	// called periodically by the Jabber class to allow us to do our own
	// processing
	function handleHeartbeat() {
		$this->beats++;
		
		// every 30 seconds, check the song title
		if ($this->beats % 30 == 0) $this->check_current_song();
	}
	
	// called when an error is received from the Jabber server
	function handleError($code,$error,$xmlns) {
		echo "Error: $error ($code)".($xmlns?" in $xmlns":"")."\n";
	}
	
	// called when a message is received from a remote contact
	function handleMessage($from,$to,$body,$subject,$thread,$id,$extended) {
		echo "Incoming message from $from to $to: $body\n";

		if ($this->bare_jid($from)==CONTROL_JID) {
			$this->parse($body,$from);
		} else {
			$this->message($from,'Sorry, I\'m not authorized to accept commands from you.');
		}

	}
	
	// utility method to strip the resource from a JID
	function bare_jid($jid) {
		list($jid,) = explode('/',$jid,2);
		return $jid;
	}
	
	// utility method to send a message to the specified JID
	function message($recipientjid,$message) {
		return $this->xmpp->message($recipientjid,"normal",NULL,$message);
	}
	
	// parses a message received from an XMPP client
	function parse($message,$jid) {
		list($cmd,$args) = explode(' ',$message,2);
		$cmd = strtolower($cmd);
		switch($cmd) {
			
			// display a message on the soundbridge screen
			case 'display':
				$lines = explode("\n",wordwrap($args,40));
				$this->message($jid,"Displaying message:\nLine 1: ".$lines[0]."\nLine 2: ".$lines[1]);
				
				// note: this blocks!  so the XMPP daemon will be unresponsive for DISPLAY_TIME seconds after
				// receiving a "display" command
				$this->soundbridge_dispatch('display',$lines[0],$lines[1],DISPLAY_TIME*1000000,true);
				break;
				
			// get the current song information
			case 'getsong':
				$song = $this->soundbridge_dispatch('get_current_song');
				$this->message($jid,'SoundBridge is currently playing track '.$song['title'].' by '.$song['artist'].' from '.$song['playlisturl'].'.');
				break;
				
			// get the current status of the soundbridge
			case 'status':
				$this->soundbridge->get_state(SOUNDBRIDGE_IP);
				$report = 'Soundbridge is ';
//				if ($soundbridge->is_playing) echo "NOW PLAYING! ";
// 				if ($soundbridge->is_paused) echo "NOW PAUSED! ";

				if ($this->soundbridge->track) $report .= "on track {$this->soundbridge->track}, and is ";
				$report .= "currently {$this->soundbridge->state}.";
				
				list($uptime,$elapsed,$volume) = $this->soundbridge_dispatch(array('uptime','get_elapsed_time','volume'));
				$report .= "  Uptime is ".floor($uptime)." seconds, and current play time is {$elapsed}.";

				$this->message($jid,$report);
				break;
				
			// change te playback state
			case 'stop':
			case 'start':
			case 'pause':
				$report = $this->soundbridge_dispatch($cmd) ? 'SoundBridge status updated.' : 'Could not execute command: '.$cmd;
				$this->message($jid,$report);
				break;
				
			// change tracks
			case 'prev':
			case 'next':
				$report = $this->soundbridge_dispatch('play_'.$cmd) ? 'SoundBridge track changed.' : 'Could not execute command: play_'.$cmd;
				$this->message($jid,$report);
				break;
				
			// display help
			case 'help':
				$report = 	"XMPP Roku SoundBridge Daemon Help\n" .
							"stop/start/pause - change play state\n" .
							"prev/next - change track\n" .
							"status - display SoundBridge status\n" .
							"getsong - display current song info\n" .
							"display [message] - displays a message on the SoundBridge screen";
				$this->message($jid,$report);
				break;
			default:
				$this->message($jid,'Sorry, I didn\'t understand that command.  Try \'help\' for assistance.');
				break;
		}
	}
	
	
	// Dispatches a command to the SoundBridge class (this just automates the process of
	// connecting, running the specified command, and disconnecting so we don't have
	// to do it by hand each time).
	//
	// The first argument can either be a single method name, or an array of method
	// names to call.  The return value will either be a single result value from the
	// method call, or an array of result values from each method call.
	function soundbridge_dispatch() {
		// grab the argument list and pull out the method list
		$args = func_get_args();
		$methodlist = array_shift($args);
		
		// connect to the soundbridge
		if (!$this->soundbridge->connect(SOUNDBRIDGE_IP)) return false;

		if (!is_array($methodlist)) $methodlist = array($methodlist);

		// call each method in order		
		$results = array();
		foreach ($methodlist as $k=>$method_name) {
			$method = array(&$this->soundbridge,$method_name);
			if (!is_callable($method)) {
				$results[$k] = false;
			} else {
				$results[$k] = call_user_func_array($method,$args);
			}
		}
		
		// disconnect
		$this->soundbridge->disconnect();
		
		if (count($methodlist)==1) $results = array_pop($results);
		
		return $results;
	}
	
	// check the current song title; if it's changed, update our XMPP status message
	function check_current_song() {
		$song = $this->soundbridge_dispatch('get_current_song');
		if ($song['title']!=$this->old_song_title) {
			$this->old_song_title = $song['title'];
			$this->xmpp->set_presence("",$song['title'].' by '.$song['artist']);
		}
	}
	
	
	
}

// create an instance of the Jabber class
$xmpp = new Jabber(false);

// create an instance of our event handler class
$test = new XMPPEventHandler($xmpp);

// set handlers for the events we wish to be notified about
$xmpp->set_handler("connected",$test,"handleConnected");
$xmpp->set_handler("authenticated",$test,"handleAuthenticated");
$xmpp->set_handler("authfailure",$test,"handleAuthFailure");
$xmpp->set_handler("heartbeat",$test,"handleHeartbeat");
$xmpp->set_handler("error",$test,"handleError");
$xmpp->set_handler("message_normal",$test,"handleMessage");
$xmpp->set_handler("message_chat",$test,"handleMessage");
$xmpp->set_handler("subscribe",$test,"handleSubscribe");

// connect to the Jabber server
if (!$xmpp->connect(XMPP_SERVER,5222,15,defined('XMPP_SERVER_IP') ? XMPP_SERVER_IP : false)) {
	die("Could not connect to XMPP server!\n");
}

// now, tell the Jabber class to begin its execution loop
$xmpp->execute(CBK_FREQ,RUN_TIME);

// disconnect from the Jabber server
$xmpp->disconnect();
?>